;	DATE: 27/2/2009
;	D.G. MORTIMER
;
;	Programme: i2c_duo.asm
;
;	reads 8 bit port and sends to i2c 
;
;	Uses 2 line hand shake to transfer 2 bytes of data from the decoder
;
;	Data can be read from the decoder independent of i2c activity
;
;	No data will be sought from the decoder until master has read and cleared data 
;	
;	addr0 = main code byte
;	addr1 = secondary code byte
;	addr2 = control byte 	
;
;	WRITTEN FOR PIC 16F88
;
;	This version written for infra red remote control and AT/PS2 keyboard
;	Last modification 24/8/2009 
;........................................................................
;
;
	#include p16f88.inc		;assembler directives
 	list p=p16f88


	keycount equ h'20'
	
	kda	equ	1 	;data from keyboard port b
	kcl	equ	0	;clock from keyboard
	scancode equ h'21'		;data read from keyboard

	index equ h'22'		;scan code position
	tablesize equ h'23'

	uSec10 equ h'24'		;delay variables
	ms1 equ h'25'
	sec1 equ h'26'
	ms10 equ h'27'

	rdy equ 7		;handshake line i/p port b
	req_ack equ 6		;handshake line o/p port b

	delay_byte equ h'28'
	databyte equ h'29' 		;register for data to o/p or i/p
	buff_size equ 8		;number of internal addresses
	offset equ h'2a'		;internal address pointer
	addr0 equ h'2b'		;internal memory locations
	addr1 equ h'2c'		;more or less as required or ram permits
	addr2 equ h'2d'
	addr3 equ h'2e'
	addr4 equ h'2f'
	addr5 equ h'30'
	addr6 equ h'31'
	addr7 equ h'32'

	i2cdata equ h'33'		;display variables
	tempw equ h'70'
	status_temp equ h'71'
	fsr_temp equ h'72'

	address equ b'00011000'	;base address of slave

	__config config1, _lvp_off & _boden_off & _mclr_off & _wdt_off & _intrc_io & _cp_off


	ORG 0			;LOCATION OF RESET VECTOR
	GOTO START	

	ORG  4			;LOCATION OF PERIPHERAL INTERRUPT VECTOR 
	GOTO INTERRUPT		;INTERRUPT ROUTINE LOCATION

	ORG 6			;LOCATION OF RESET START PROGRAMME

start:
	clrf porta		;get rid of any leftover data
	clrf portb
	bcf status,irp
	bcf status,rp1
	bsf status,rp0		;bank 1
	clrf ansel
	
	movlw h'ff'
	movwf trisa		;port a inputs
	movlw b'10111111'
	movwf trisb		;port b inputs b6 output

	bsf trisb,1		;scl and sda as inputs for i2c
	bsf trisb,4		;to be doubly sure
	movlw b'01100000'	;set int rc osc to 4MHz
	movwf osccon
	clrf sspstat		;clear ssp status register
	bcf status,rp0		;select bank 0
	movf portb,w
	andlw b'00001100'	;mask for address bits
	movwf tempw		;save bits in temp register
	bcf status,c		;clear carry bit
	rrf tempw,w		;shit bits to pos 1 and 2
	iorlw address		;combine with base address
	bsf status,rp0		;bank 1
	movwf sspadd
	bsf pie1,sspie		;enable interrupts
	bcf status,rp0		;bank 0
	bcf pir1,sspif
	movlw b'00110110'	;set up for slave read no interrupt
	movwf sspcon
	bsf intcon,peie		
	bsf sspcon,sspen
	bsf intcon,gie

;===================================================================
;                    end of pic setup routine
;  		beginning of main-loop initialisation
;===================================================================
	clrf offset		;set up indirect address for slave regs


;===================================================================
;===================================================================
;       main slave function programme starts here
;===================================================================
;===================================================================
	bcf portb,req_ack
	movlw 5
	call delayms		;delay 1 second
	clrf addr0	
	clrf addr1
	clrf addr2
	
	clrf i2cdata
	clrf scancode
	clrf porta
main_loop:			;not used here
	
wait_data:

	movlw 0
	subwf addr2,w		;wait till master is ready
	btfss status,z
	goto wait_data		;no data yet
	clrf addr0
	clrf addr1
	clrf addr2
	bsf portb,req_ack
	
wait_rdy_hi:
	btfss portb,rdy		;wait for valid data signal from decoder
	goto wait_rdy_hi
	movf porta,w
	movwf addr0		;get and save main code byte
	bcf portb,req_ack	;tell decoder to send next byte
wait_rdy_lo:
	btfsc portb,rdy		;wait for valid data signal from decoder
	goto wait_rdy_lo
	movf porta,w
	movwf addr1		;get and save secondary code byte
	movlw 1
	movwf addr2		;indicate to master good data
	goto wait_data		;back to beginning

;===================================================================

delay10:			;10uSec per loop
	movwf usec10		;come in with uSx10 count in w reg
delay10a:
	nop			;pad out to 10uSecs
	nop
	nop
	nop
	nop
	nop
	decfsz usec10,f
	goto delay10a		;loop count 10uSec loops
	return

delayms:
	movwf ms1		;come in with mSec count in W reg
delaymsa:
	movlw 100		;set up for 100 x 10uSec = 1 mSec
	call delay10
	decfsz ms1,f
	goto delaymsa
	return

delayms10:			;come in with mSx10 count in W reg
	movwf ms10
delayms10a:
	movlw 10
	call delayms
	decfsz ms10,f
	goto delayms10a
	return

delaysec:
	movwf sec1		;come in with seconds count in W reg
delayseca:
	movlw 100		;1000 mSec
	call delayms10			
	decfsz sec1,f
	goto delayseca
	return

;========================================================================
;	End of Main Loop Routines etc.
;	Beginning of interrupt routines
;========================================================================
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;========================================================================
;	this is the data i/o section - i2c slave main routine
;========================================================================
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

interrupt:
	bcf status,rp0		bank 0
	bcf pir1,sspif
	movwf tempw		;save contents of w reg
	movf status,w
	movwf status_temp	;save contents of status reg
	movf fsr,w
	movwf fsr_temp		;save fsr register
	movf sspbuf,w		;clear data buffer 
	movwf databyte		;save in databyte
	bsf status,rp0		;bank 1
	btfsc sspstat,d_a	;check if data or address
	goto data_mode

;========================================================================
;	get and process mode byte	
;========================================================================

	btfsc sspstat,r_w	;check if we need a mode byte
	goto send
	call inbyte		;read mode byte
	movlw buff_size		;get number of internal addresses
	subwf databyte,w	;check if valid address
	btfsc status,c		;valid if carry is 0 
	clrf databyte		;if bad address, assume address 0
	movf databyte,w		;good address, so put it in <offset>
	movwf offset
	goto int_end

data_mode:

	btfss sspstat,r_w	;check if read or write
	goto receive		;input data
send:
	bcf status,rp0		;bank 0
	movlw addr0		;physical location of address block	
	addwf offset,w		;point to required address
	movwf fsr		;put address in indirect address mode
	movf indf,w		;get data to be output
	btfsc sspcon,ckp	;check if another byte
	goto int_end		;no so finish
	movwf sspbuf		;send data to master
	nop			;delay needed to correct timing
	nop
	nop
	nop
	nop
	bsf sspcon,ckp
	bcf pir1,sspif
	incf offset,f
	movlw buff_size
	subwf offset,w
	btfsc status,z
	clrf offset
	goto int_end

receive:
	bcf status,rp0		;bank 0
	movlw addr0		;physical location of address block
	addwf offset,w		;point to required address
	movwf fsr		;put address in indirect address mode
	movf databyte,w
	movwf indf		;put data in appropriate address
	incf offset,f		;automatic increment of address
	movlw buff_size
	subwf offset,w		;check if exceeded address limit
	btfsc status,z
	clrf offset		;back to start of addresses
	goto int_end	

;========================================================================

int_end:


	bcf status,rp0		;reset bank 0
	movf fsr_temp,w
	movwf fsr		
	movf status_temp,w	;restore temps
	movwf status
	movf tempw,w
	bcf pir1,sspif		;clear the interrupt flag
	retfie  		

inbyte:
	bcf status,rp0		;bank 0
	btfss pir1,sspif	;wait for databyte in
	goto inbyte
	movf sspbuf,w		;read data buffer and put in databyte reg
	movwf databyte
	bcf pir1,sspif		;clear interrupt flag
	return	

        end			;END OF PROGRAMME - MANDATORY COMMAND
